package ca.ubc.cs.spl.pattern.library;

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the design patterns project at UBC
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ca.ubc.cs.spl.patterns.
 *
 * Contributor(s):   
 */

import java.util.WeakHashMap;

/**
 * Defines the general behavior of the chain of responsibility design pattern.
 *
 * Each concrete sub-aspect of this aspect defines one particular instance of
 * the pattern, with an arbitrary number of <i>Handler</i> classes involved.
 *
 * The sub-aspect defines three things: <ol>
 *
 *   <li> what types can be handlers
 *
 *   <li> what triggers an event to be raised
 *
 *   <li> how each <i>Handler</i> treats a particular event. The default
 *        implementation in this aspect here makes it only necessary to code
 *        those cases where an event is actually handled.
 * </ol>
 *
 * Note that in this implementation, a default behavior for <i>Handler</i>s is
 * defined: unless a <i>Handler</i> explicitly handles an event, it is passed
 * to its successor. In addition to that, the implementation allows to define
 * a default bahavior in case no <i>Handler</i> in the chain handles an event.
 * Here, an error message is shown.  <p>
 *
 * Note further that this version does not define a <i>Request</i> role. 
 * Although that would be a more consistent approach, AspectJ does not allow
 * the mechanisms used here if the code is not accessible by <code>ajc</code>.
 * Thus, the pointcuts would not match appropriately. A solution would be to 
 * Subclass all potential Objects, but that seems to be unneccessarily
 * complex.
 *
 * @author  Jan Hannemann
 * @author  Gregor Kiczales
 * @version 1.0, 05/13/02
 *
 */

public abstract aspect ChainOfResponsibilityProtocol {
    
    /** 
     * true if the notification mode is on, false otherwise
     */  
    
    protected boolean notificationOn = false;
    
    /**
     * This interface is used by extending aspects to say what types
     * handle requests. It models the <i>Handler</i> role.
     */

	protected interface Handler {}    


    /**
     * Stores the mapping between <code>Handler</code>s and its<i>
     * Successor</i>. For each handler, its <i>Successor</i>
     * is stored.
     */
	
	public WeakHashMap successors = new WeakHashMap();
	
    
    /**
     * Implements the abstracted CoR behavior. Assumption: If the request is
     * not handled, the last receiver handles it by default ( that is true
     * for this implementnation of the protocol). 
     *
     * The current handler gets asked if it wants to handle the request. If
     * not, the request is forwarded to its successor
     */

	protected void receiveRequest(Handler handler, Object request) {          
		note("Call received  for: "+handler.getClass().getName());
		if (handler.acceptRequest(request)) {
  		    note("Class "+handler.getClass().getName()+" is accepting the request");
		    handler.handleRequest(request);
  		    note("Call handled.\n");
		} else {    
  		    note("Class "+handler.getClass().getName()+" is rejecting the request");
  		    Handler successor = getSuccessor(handler);         
		    if (successor == null) {
		         note("Request unhandled!\n");
		    } else {
       			note("Call forwarded to: "+successor.getClass().getName());
   	    	    receiveRequest(successor, request);
   	    	}
		}
	}  

    /**
     * Method attached to Handlers: returns true if a Handler wants to hanlde 
     * that request. By default, requests are rejected.
     *
     * @param request the request to be handled
     */
		
	public boolean Handler.acceptRequest(Object request) {  
	    System.out.println("Default reject: event from "+this.getClass().getName());
	    return false;
    }
	
    /**
     * Method attached to Handlers: handles a request. Default implementation
     * does nothing, only shows an error message.
     *
     * @param request the request to be handled
     */
		
	public void Handler.handleRequest(Object request) {          // The Event is not handled elsewhere
	    System.out.println("No successor, request ignored");
	}


    /**
     * The join points after which a request is raised.
     * It replaces the normally scattered calls to <i>notify()</i>. To be
     * concretized by sub-aspects.
     */ 
     
    protected abstract pointcut eventTrigger(Handler handler, Object request); 
    

    /**
     * Call receiveRequest after a request was triggered.
     *
     * @param s the subject on which the change occured
     */

	after(Handler handler, Object request): eventTrigger(handler, request) { 
		receiveRequest(handler, request);
	}
 
 
    /**
     * Adds a successor to a handler.  
     *
     * @param handler the handler to attach a new successor to
     * @param successor the new successor to attach
     */ 
     
	public void setSuccessor(Handler handler, Handler successor) {
		successors.put(handler, successor);
	}  

	
    /**
     * Returns the successor of a handler.  
     *
     * @param handler the handler in question
     * @return the successor of the handler
     */ 
     
	public Handler getSuccessor(Handler handler) { 
	    return ((Handler) successors.get(handler));
	} 

	
    /**
     * Turns the notification mode on or off.   
     *
     * @param choice true to turn notifications on, false to turn them off 
     */ 
     
	public void setNotification(boolean choice) { 
	    this.notificationOn = choice;
	} 	

    /**
     * Returns the successor of a handler.  
     *
     * @param msg the message to print
     */ 
     
	public void note(String msg) { 
	    if (notificationOn) {
	        System.out.println(msg);
	    }
	} 
}