package ca.ubc.cs.spl.pattern.library;

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the design patterns project at UBC
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ca.ubc.cs.spl.patterns.
 *
 * Contributor(s):   
 */

import java.util.Hashtable;

/**
 * Defines the general behavior of the singleton design pattern.
 *
 * Each concrete sub-aspect of SingletonProtocol defines the Singleton
 * property for one class.
 *
 * The sub-aspect defines two things: <ol>
 *
 *   <li> what type is a <i>Singleton</i> <br>
 *
 *   <li> what classes can access the <i>Singleton</i>'s constructor (if any)
 *        despite its property
 * </ol>
 *
 * for this implementation we choose to illustrate that it is not necessary
 * to provide a factory method for (like getSingleton()), but the regular
 * constructor may be used instead to access the Singleton instance
 *
 * @author  Jan Hannemann
 * @author  Gregor Kiczales
 * @version 1.0, 05/13/02
 *
 */
  
public abstract aspect SingletonProtocol {
	
    private Hashtable singletons = new Hashtable(); 

    public interface Singleton {}
    
    protected pointcut protectionExclusions();                                        // What if two aspects re-define protectionExclusions() ?
                                                                                                                            
	Singleton around(): call((Singleton+).new(..)) && !protectionExclusions() {                    
	    Class singleton = thisJoinPoint.getSignature().getDeclaringType(); 
		if (singletons.get(singleton) == null) {                            // How to access the static instance variable here?
		    singletons.put(singleton, proceed()); 
		}
		return (Singleton) singletons.get(singleton);
	} 
	
	
	// Alternative to protectionExclusions() ?:
	// 
	// Define an interface NonSingleton and check in advice whether 
	// we are asked to create a NonSingleton, then proceed
}