package ca.ubc.cs.spl.pattern.library;

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This file is part of the design patterns project at UBC
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * either http://www.mozilla.org/MPL/ or http://aspectj.org/MPL/.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ca.ubc.cs.spl.patterns.
 *
 * Contributor(s):   
 */

import org.aspectj.lang.JoinPoint; 

/**
 * Defines the abstracted Proxy design pattern.<p>  
 *
 * Concrete sub-aspects define the following: <UL>
 *  <LI> Which class(es) are subjects
 *  <LI> Which accesses should be protected (methods, field accesses)
 *  <LI> What alternative value should be returned in case of a denied
 *       access to a method.
 * </UL>
 *
 * @author  Jan Hannemann
 * @author  Gregor Kiczales
 * @version 1.0, 05/13/02
 * 
 */    
 
public abstract aspect ProxyProtocol { 
    
    /**
     * Defines the Subject role (for correct typing)
     */
    
    protected interface Subject {}  
    
    /**
     * Captures all accesses to the subject that should be protected by 
     * this pattern instance.
     */
                                        
	protected abstract pointcut protectedAccess();     
	
	/** 
	 * Extends the <code> protectedAccess()</code> pointcut to include a 
	 * field for the joinpoint. Used internally only.
	 */
	
	private pointcut accessByCaller(Object caller): 
	    protectedAccess() && this(caller);

    /**
     * Intercepts accesses to protected parts of the Subject. If access is 
     * rejected, the method <code>handleFailedAccess(..)</code> is called
     * instead.
     *
     * @param caller the object responsible for the protected access
     * @param subject the subject receiving the call
     */

	Object around(Object caller, Subject subject): 
	    accessByCaller(caller) && target(subject) { 
	        
		if (! rejection(caller, subject, thisJoinPoint) )
			return proceed(caller, subject); 
		return handleFailedAccess(caller, subject, thisJoinPoint);
	} 
	
	/**
	 * Checks whether the access should be denied or not
	 *
     * @param caller the object responsible for the protected access
     * @param subject the subject receiving the call  
     * @param joinPoint the joinpoint associated with the protected access
     *
     * @returns true if the access is denied, false otherwise
     */
	
	protected abstract boolean rejection(Object caller, 
	                                     Subject subject, 
	                                     JoinPoint joinPoint);

	/**
	 * For delegation: Provides an alternative return value if access 
	 * is denied.
	 *
     * @param caller the object responsible for the protected access
     * @param subject the subject receiving the call  
     * @param joinPoint the joinpoint associated with the protected access
     *
     * @returns an alternative return value
     */

	protected abstract Object handleFailedAccess(Object caller, 
	                                             Subject subject, 
	                                             JoinPoint joinPoint);
} 
